/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.general;

/**
 * Main compass directions.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public enum Direction
{
    NORTH
        {
            public Point getNewPosition(Point origPosition)
            {
                return new Point(origPosition.getX(), origPosition.getY() - 1);
            }
        },


    EAST
        {
            public Point getNewPosition(Point origPosition)
            {
                return new Point(origPosition.getX() + 1, origPosition.getY());
            }
        },

    SOUTH
        {
            public Point getNewPosition(Point origPosition)
            {
                return new Point(origPosition.getX(), origPosition.getY() + 1);
            }
        },

    WEST
        {
            public Point getNewPosition(Point origPosition)
            {
                return new Point(origPosition.getX() - 1, origPosition.getY());
            }
        };

    /**
     * @assume origPosition != null
     * @return the new position after walking one step in the given direction
     */
    public abstract Point getNewPosition(Point origPosition);
}
