/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.general;

import java.util.LinkedList;

/**
 * An Observable with extra typing.
 */
public class Observable<V, O extends Observer<V>>
{
    private LinkedList<O> observers;

    public Observable()
    {
        this.observers = new LinkedList<O>();
    }

    public void addObserver(O observer)
    {
        this.observers.add(observer);
    }

    public void notifyObservers(V value)
    {
        for (O observer : this.observers)
            observer.update(value);
    }

    public int countObservers()
    {
        return this.observers.size();
    }

    public void deleteObservers()
    {
        this.observers.clear();
    }
}

