/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.general;

/**
 * A point in a 300x300 plane.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class Point
{
    private int x;
    private int y;

    public Point(int x, int y)
    {
        this.x = x;
        this.y = y;
    }

    public int getX()
    {
        return this.x;
    }

    public int getY()
    {
        return this.y;
    }

    /**
     * Utilizes that two points are equal if they share the same x and y
     * values.
     * @return the equality
     */
    @Override
    public boolean equals(Object obj)
    {
        if (obj == null || !(obj instanceof Point))
            return false;
        Point otherPoint = (Point) obj;
        return (otherPoint.getX() == this.x && otherPoint.getY() == this.y);
    }

    /**
     * Generates a hash code.
     *
     * Dungeon crawler requires at most 300x300 = 90000 points, which is well
     * below 2^31, which means we can make a unique hash code for every
     * (x,y)-pair.
     * @return the unique hash code 
     */
    @Override
    public int hashCode()
    {
        return this.x + this.y * 300;
    }

    @Override
    public String toString()
    {
        return "(" + this.x + ", " + this.y + ")";
    }
}
    