/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.gui.component;
import dungeoncrawl.gui.*;
import dungeoncrawl.model.*;
import dungeoncrawl.model.item.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Item to draw.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class ItemComponent extends DungeonComponent
{
    private Item item;
    private Color color;
    private int diameter;
    private int radius;
    
    public ItemComponent(Item item, DungeonPanel dungeonPanel)
    {
        super(dungeonPanel);
        this.item = item;
        this.createData();
    }

    public void createData()
    {
        this.color = (item instanceof Armor) ? Color.YELLOW : Color.MAGENTA;        
        this.diameter = this.dungeon.getScaleFactor() / 2;
        this.radius = this.dungeon.getScaleFactor() / 4;
    }

    // Draws the item
    @Override
    public void paintComponent(Graphics g)
    {
        int x = (this.item.getPosition().getX()
                 * this.dungeon.getScaleFactor()
                 + this.radius);
        int y = (this.item.getPosition().getY()
                 * this.dungeon.getScaleFactor()
                 + this.radius);
        g.setColor(this.color);
        g.drawArc(x, y, this.diameter, this.diameter, 0, 360);
    }
}
