/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.gui.component;
import dungeoncrawl.gui.*;

import java.util.Random;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A text field with a start value that disappears once you click on the
 * field. Subsequent values do not disappear if you click on the field.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class RandomStartValueTextField extends JTextField
{
    public RandomStartValueTextField(int width, String[] startValues)
    {
        super(width);
        Random random = new Random();
        this.setText(startValues[random.nextInt(startValues.length)]);
        this.addMouseListener(new ClickListener(this));
    }

    private class ClickListener extends MouseAdapter
    {
        private RandomStartValueTextField field;

        public ClickListener(RandomStartValueTextField field)
        {
            this.field = field;
        }

        @Override
        public void mouseClicked(MouseEvent e)
        {
            this.field.setText("");
            this.field.removeMouseListener(this);
        }
    }
}
