/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.gui.controller;
import dungeoncrawl.gui.*;
import dungeoncrawl.general.*;
import dungeoncrawl.model.*;
import dungeoncrawl.model.player.hero.Hero;

import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import javax.swing.SwingUtilities;

/**
 * The key listener for the game.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class DungeonKeyListener extends KeyAdapter
{
    private final DungeonPanel dungeonPanel;
    private final Dungeon dungeon;
    private final Hero hero;
    
    public DungeonKeyListener(DungeonPanel dungeonPanel)
    {
        this.dungeonPanel = dungeonPanel;
        this.dungeon = this.dungeonPanel.getDungeon();
        this.hero = this.dungeon.getHero();
    }
    
	@Override
	public void keyReleased(KeyEvent keyEvent)
	{
        if (this.dungeonPanel.isFinished())
            {
                if (keyEvent.getKeyCode() == KeyEvent.VK_SPACE)
                    this.dungeonPanel.end();
                return;
            }
		switch(keyEvent.getKeyCode())
            {
            case KeyEvent.VK_UP:
                this.tryMove(Direction.NORTH);
                break;
            case KeyEvent.VK_RIGHT:
                this.tryMove(Direction.EAST);
                break;
            case KeyEvent.VK_DOWN:
                this.tryMove(Direction.SOUTH);
                break;
            case KeyEvent.VK_LEFT:
                this.tryMove(Direction.WEST);
                break;
            case KeyEvent.VK_I:
                this.toggleShowingStatus();
                break;
		}
	}

    private void toggleShowingStatus()
    {
        this.dungeonPanel.toggleShowingStatus();
    }

    private void tryMove(Direction direction)
    {
        if (this.dungeonPanel.isShowingStatus())
            return;
        final Direction finalDirection = direction;
        SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    hero.tryMove(finalDirection);
                }
            });
    }
}
