/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.model;
import dungeoncrawl.general.*;
import java.util.HashMap;
import java.util.Random;

/**
 * A door. A door only needs a position. It is not necessary to store a link to
 * the room the door leads into, as a door by definition is at the wall and
 * leads into the room behind that wall. If one wanted to implement portals,
 * something more would be needed. A Door can be thought of as a hole in the
 * wall.
 * 
 * This Door class does keep information about its target room, but that is
 * just because it is useful, not because it is necessary.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class Door
{
    private Point position;
    private Room room;
    private Point targetPosition;
    private Room targetRoom;
    
    /**
     * Creates a door in a room with a target room.
     * @require position != null
     * @require targetPosition != null
     * @require room != null
     * @require targetRoom != null
     */
    public Door(Point position, Room room, Point targetPosition,
                Room targetRoom)
    {
        if (position == null || room == null || targetPosition == null
            || targetRoom == null)
            {
                throw new NullPointerException();
            }
        this.position = position;
        this.room = room;
        this.targetPosition = targetPosition;
        this.targetRoom = targetRoom;
    }

    public Point getPosition()
    {
        return this.position;
    }

    public Room getRoom()
    {
        return this.room;
    }

    public Point getTargetPosition()
    {
        return this.targetPosition;
    }

    public Room getTargetRoom()
    {
        return this.targetRoom;
    }
}
