/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.model.item;
import dungeoncrawl.general.*;
import dungeoncrawl.model.*;
import dungeoncrawl.model.player.hero.Hero;

/**
 * Some armor. No Armor can have a negative resistance. There is no limit the
 * the amount of armors that can be worn at the same time.
 * @author Niels G. W. Serup
 * @version 1.0
 * @invariant this.resistance >= 0
 */
public class Armor extends Item
{
    private int resistance;

    /**
     * Creates a piece of armor with {@code resistance} resistance and with a
     * position.
     */
    public Armor(Dungeon dungeon, int resistance, Point position)
    {
        super(dungeon, position);
        this.resistance = resistance;
    }

    /**
     * Creates a piece of armor with {@code resistance} resistance and with no
     * position.
     */
    public Armor(Dungeon dungeon, int resistance)
    {
        this(dungeon, resistance, null);
    }

    /**
     * Creates a piece of armor with 0 resistance and no position.
     */
    public Armor(Dungeon dungeon)
    {
        this(dungeon, 0, null);
    }

    public int getResistance()
    {
        return this.resistance;
    }

    public void increaseResistance(int moreResistance)
    {
        this.resistance += moreResistance;
    }

    public void decreaseResistance(int lessResistance)
    {
        this.resistance -= lessResistance;
    }

    public void setResistance(int resistance)
    {
        this.resistance = resistance;
    }

    public void getUsedByHero(Hero hero)
    {
        hero.pickUpArmor(this);
    }

    @Override
    public String toString()
    {
        return "A" + this.resistance;
    }
}
