/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.model.item;
import dungeoncrawl.model.*;
import dungeoncrawl.general.*;
import dungeoncrawl.model.player.hero.Hero;


/**
 * An item.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public abstract class Item extends DungeonObject
{
    /**
     * Creates an item associated with {@code dungeon}. {@code position} can be
     * null, but remember to set it later with {@code setPosition}.
     * @require dungeon != null
     */
    public Item(Dungeon dungeon, Point position)
    {
        super(dungeon, position);
    }

    /**
     * Creates an item associated with {@code dungeon} and with no position.
     * @require dungeon != null
     */
    public Item(Dungeon dungeon)
    {
        this(dungeon, null);
    }

    /**
     * An item cannot be placed on other items.
     */
    @Override
    public boolean isPositionGood(Point position)
    {
        return (this.getDungeon().getItemAt(position) == null);
    }

    
    /**
     * What happens when the hero loots the item.
     */
    public abstract void getUsedByHero(Hero hero);
}
