/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.model.movementstrategy;
import dungeoncrawl.general.*;
import dungeoncrawl.model.player.monster.Monster;
import dungeoncrawl.model.*;

import java.util.Random;

/**
 * Move towards the shortest Manhattan distance.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class ManhattanMovementStrategy extends MovementStrategy
{
    public ManhattanMovementStrategy()
    {
        super();
    }

    public ManhattanMovementStrategy(Monster monster)
    {
        super(monster);
    }
    
    /**
     * Moves a monster towards the shortest Manhattan distance between it and
     * the hero. When the same Manhattan distance can be reached by both a y
     * movement and an x movement, one is chosen at random.
     */
    @Override
    public void move(Point target)
    {
        Point monsterPos = this.monster.getPosition();
        if (new Random().nextBoolean())
            {
                if (!this.moveOnXAxis(monsterPos, target))
                    this.moveOnYAxis(monsterPos, target);
            }
        else
            {
                if (!this.moveOnYAxis(monsterPos, target))
                    this.moveOnXAxis(monsterPos, target);
            }
        // if monsterPos.equals(target), do nothing.
    }

    // Returns true if the monster might (if one ignores walls) be able to
    // reduce the Manhattan distance and actually moves; false otherwise.
    private boolean moveOnXAxis(Point monsterPos, Point target)
    {
        if (monsterPos.getX() < target.getX())
            {
                monster.tryMove(Direction.EAST);
                return true;
            }
        if (monsterPos.getX() > target.getX())
            {
                monster.tryMove(Direction.WEST);
                return true;
            }
        return false;
    }

    // Does the same as moveOnXAxis, just on the y axis
    private boolean moveOnYAxis(Point monsterPos, Point target)
    {
        if (monsterPos.getY() < target.getY())
            {
                monster.tryMove(Direction.SOUTH);
                return true;
            }
        if (monsterPos.getY() > target.getY())
            {
                monster.tryMove(Direction.NORTH);
                return true;
            }
        return false;
    }

    @Override
    public int getTargetApproximity(Point target)
    {
        return this.getManhattanDistance(target);
    }

    protected int getManhattanDistance(Point target)
    {
        Point monsterPos = this.monster.getPosition();
        return (Math.abs(monsterPos.getX() - target.getX())
                + Math.abs(monsterPos.getY() - target.getY()));
    }
}
