/*
 * Dungeon Crawl of Doom
 * Copyright (C) 2012  Niels G. W. Serup

 * This file is part of Dungeon Crawl of Doom.

 * Dungeon Crawl of Doom is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.

 * Dungeon Crawl of Doom is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */


package dungeoncrawl.model.player.monster;
import dungeoncrawl.general.*;
import dungeoncrawl.model.player.Player;
import dungeoncrawl.model.*;
import dungeoncrawl.model.movementstrategy.*;

/**
 * An Orc.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class Orc extends Monster
{
    private static final int REGENERATION_LEVEL = 10;
    private static final int DAMAGE_LEVEL = 20;
    
    /**
     * Creates an Orc associated with {@code dungeon}.
     */
    public Orc(Dungeon dungeon, Point position,
               MovementStrategy movementStrategy)
    {
        super(dungeon, position, movementStrategy);
        this.setRegenerationLevel(Orc.REGENERATION_LEVEL);
    }

    /**
     * Creates an Orc associated with {@code dungeon} with a default
     * movement strategy.
     */
    public Orc(Dungeon dungeon, Point position)
    {
        this(dungeon, position, null);
    }

    /**
     * Creates an Orc associated with {@code dungeon} with a default
     * movement strategy and no position.
     */
    public Orc(Dungeon dungeon)
    {
        this(dungeon, null, null);
    }

    /**
     * Gets the damage level.
     * @ensure return >= 0.
     * @return the level of damage that the monster can incur on another player.
     */
    @Override
    public int getDamageLevel() {
        return Orc.DAMAGE_LEVEL;
    }
}