// njf941

import dungeoncrawl.model.*;
import dungeoncrawl.general.*;
import dungeoncrawl.model.player.Player;
import dungeoncrawl.model.player.monster.*;
import dungeoncrawl.model.player.hero.*;
import dungeoncrawl.model.item.*;


import static org.junit.Assert.*;
import org.junit.Test;

import java.util.ArrayList;

/**
 * Testing of Dungeon and Dungeon-related classes who exist in dungeons.
 * 
 * I do not test movement strategies, as they can be hard to predict.
 * 
 * The attack system is too simple for me to want to spend time writing tests
 * for it.
 * @author Niels G. W. Serup
 * @version 1.0
 */
public class DungeonTests {
    private Dungeon dungeon;
    private Hero hero;
    private Direction n = Direction.NORTH;
    private Direction e = Direction.EAST;
    private Direction s = Direction.SOUTH;
    private Direction w = Direction.WEST;
    
    /**
     * Tests if it is possible to walk through doors and impossible to walk
     * through walls.
     * 
     * Uses these start conditions:
     * where H := the hero
     *       D := a door
     *       / := room 1
     *       \ := room 2
     *       0 := nothingness
     *
     * There is of course a wall between room 1 and room 2 (it's just not drawn
     * in the example).
     * 
     * 000000000000000000000
     * 0//////////0000000000
     * 0/H////////0000000000
     * 0//////////\\\\\\\\\\
     * 0//////////\\\\\\\\\\
     * 0//////////\\\\\\\\\\
     * 0//////////\\\\\\\\\\
     * 0/////////DD\\\\\\\\\
     * 0//////////\\\\\\\\\\
     * 0//////////\\\\\\\\\\
     * 00000000000\\\\\\\\\\
     * 00000000000\\\\\\\\\\
     * 00000000000\\\\\\\\\\
     * 00000000000\\\\\\\\\\
     */
    @Test
    public void testWallsDoors()
    {
        this.testWallsDoorsSetup();
        
        mv(n, 2, 1);
        mv(n, 2, 1);
        mv(w, 1, 1);
        mv(w, 1, 1);
        mv(n, 1, 1);
        mv(s, 1, 2);
        mv(s, 1, 3);
        mv(s, 1, 4);
        mv(w, 1, 4);
        mv(e, 2, 4);
        mv(e, 3, 4);
        mv(n, 3, 3);
        mv(s, 3, 4);
        mv(s, 3, 5);
        mv(s, 3, 6);
        mv(e, 4, 6);
        mv(e, 5, 6);
        mv(s, 5, 7);
        mv(s, 5, 8);
        mv(s, 5, 9);
        mv(s, 5, 9);
        mv(e, 6, 9);
        mv(e, 7, 9);
        mv(e, 8, 9);
        mv(e, 9, 9);
        mv(e, 10, 9);
        mv(e, 10, 9);
        mv(e, 10, 9);
        mv(w, 9, 9);
        mv(n, 9, 8);
        mv(e, 10, 8);
        mv(n, 10, 7);
        mv(n, 10, 6);
        mv(n, 10, 5);
        mv(e, 10, 5);
        mv(s, 10, 6);
        mv(e, 10, 6);
        mv(s, 10, 7);
        mv(s, 10, 8);
        mv(e, 10, 8);
        mv(n, 10, 7);
        mv(e, 11, 7); // Going through door from / to \.
        mv(w, 10, 7); // Going through door from \ to /.
        mv(e, 11, 7); // Going through door from / to \.
        mv(n, 11, 6);
        mv(w, 11, 6);
        mv(s, 11, 7);
        mv(s, 11, 8);
        mv(w, 11, 8);
        mv(s, 11, 9);
        mv(s, 11, 10);
        mv(s, 11, 11);
        mv(s, 11, 12);
        mv(s, 11, 13);
        mv(s, 11, 13);
        mv(e, 12, 13);
        mv(e, 13, 13);
        mv(e, 14, 13);
        mv(e, 15, 13);
        mv(w, 14, 13);
        mv(s, 14, 13);
        mv(e, 15, 13);
        mv(e, 16, 13);
        mv(n, 16, 12);
        mv(e, 17, 12);
        mv(e, 18, 12);
        mv(e, 19, 12);
        mv(e, 20, 12);
        mv(e, 20, 12);
        mv(n, 20, 11);
        mv(n, 20, 10);
        mv(n, 20, 9);
        mv(n, 20, 8);
        mv(n, 20, 7);
        mv(w, 19, 7);
        mv(n, 19, 6);
        mv(e, 20, 6);
        mv(s, 20, 7);
        mv(n, 20, 6);
        mv(n, 20, 5);
        mv(n, 20, 4);
        mv(n, 20, 3);
        mv(n, 20, 3);
        mv(w, 19, 3);
        mv(n, 19, 3);
    }

    private void testWallsDoorsSetup()
    {
        Dungeon dungeon = new Dungeon();
        dungeon.addRoom(new Room(dungeon,
                                 new Rectangle(new Point(1, 1),
                                               new Point(11, 10))));
        dungeon.addRoom(new Room(dungeon,
                                 new Rectangle(new Point(11, 3),
                                               new Point(21, 14))));
        dungeon.addDoor(new Point(10, 7), new Point(11, 7));
        assertFalse(dungeon.isTooLarge());
        assertFalse(dungeon.isMalformed());
        Hero hero = new Cleric(dungeon, "Aerith");
        dungeon.setHero(hero);
        hero.setPosition(new Point(2, 2));
        assertEquals(hero.getPosition(), new Point(2, 2));
        this.dungeon = dungeon;
        this.hero = hero;
    }


    /**
     * Tests if item looting functions.
     * 
     * Uses these start conditions:
     * where H := the hero
     *       / := the room
     *       A := armor 1
     *       B := armor 2
     *       W := weapon 1
     *       V := weapon 2
     *
     * /B////
     * //////
     * //A///
     * //////
     * /WV//H
     */
    @Test
    public void testItems()
    {
        this.testItemsSetup();

        mv(w, 4, 4);
        mv(w, 3, 4);
        mv(w, 2, 4); // Hit V
        assertEquals(this.hero.getWeapon().getDamage(), 6);
        assertEquals(this.hero.getDamageLevel(), 6 * Mage.DAMAGE_MAGNITUDE);
        mv(n, 2, 3);
        mv(s, 2, 4);
        assertEquals(this.hero.getWeapon().getDamage(), 6);
        assertEquals(this.hero.getDamageLevel(), 6 * Mage.DAMAGE_MAGNITUDE);
        mv(n, 2, 3);
        mv(n, 2, 2); // Hit A
        assertEquals(this.hero.getArmor().getResistance(), 40);
        mv(s, 2, 3);
        mv(n, 2, 2);
        assertEquals(this.hero.getArmor().getResistance(), 40);
        mv(w, 1, 2);
        mv(s, 1, 3);
        mv(s, 1, 4); // Hit W
        assertEquals(this.hero.getWeapon().getDamage(), 200);
        assertEquals(this.hero.getDamageLevel(), 200 * Mage.DAMAGE_MAGNITUDE);
        mv(n, 1, 3);
        mv(n, 1, 2);
        mv(n, 1, 1);
        mv(n, 1, 0);
        assertEquals(this.hero.getArmor().getResistance(), 240);
    }

    private void testItemsSetup()
    {
        Dungeon dungeon = new Dungeon();
        dungeon.addRoom(new Room(dungeon, new Point(0, 0), new Point(6, 5)));
        dungeon.addItem(new Armor(dungeon, 40, new Point(2, 2)));   // A
        dungeon.addItem(new Armor(dungeon, 200, new Point(1, 0)));  // B
        dungeon.addItem(new Weapon(dungeon, 200, new Point(1, 4))); // W
        dungeon.addItem(new Weapon(dungeon, 6, new Point(2, 4)));   // V
        assertFalse(dungeon.isTooLarge());
        assertTrue(dungeon.isMalformed()); // it has only one room
        Hero hero = new Mage(dungeon, "Bilbo");
        dungeon.setHero(hero);
        hero.setPosition(new Point(5, 4));
        assertEquals(hero.getPosition(), new Point(5, 4));
        this.dungeon = dungeon;
        this.hero = hero;
    }

    
    
    // Convenience methods
    
    private void mv(Direction direction, int endX, int endY)
    {
        this.hero.tryMove(direction);
        assertEquals(this.hero.getPosition(), new Point(endX, endY));
    }
}
